import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/material.dart';
import 'package:modern_form/modern_form_core.dart';
import 'package:modern_form/modern_form_files.dart';
import 'package:rxdart/rxdart.dart';
import 'package:youtube_explode_dart/youtube_explode_dart.dart';

class ModernFormVideoPlatformHelper {
  static Future<void> shareVideo(Uint8List bytes, String? text,
      {String? title}) async {
    await ModernFormShareHelper.file(
      title ?? '',
      'video.mp4',
      bytes.toList(),
      'video/mp4/mp3',
      text: text ?? "",
    );
  }

  static Future<Uint8List?> downloadFromYoutube(String url,
      [void Function(int, int)? onBytesReceived, bool? force]) async {
    String filePath = await ModernFormFileHelper.getFilePathFromUrl(url, "mp4");
    File file = File(filePath);
    try {
      ModernFormProgressDialog? progressDialog;

      if (onBytesReceived == null) {
        progressDialog = ModernFormProgressDialog();
        progressDialog.showProgressDialog(
          textToBeDisplayed: 'Fazendo download do vídeo...',
          progressColor: Colors.white,
          barrierColor: Color(0x88222222),
        );
      }

      bool exists = await file.exists();
      bool isForce = force != null && force;
      if (isForce && exists) {
        await file.delete();
        file = await file.create();
      }

      if (!exists || isForce) {
        YoutubeExplode yt = YoutubeExplode();
        MuxedStreamInfo streamInfo = await getStreamInfo(yt, url);

        Stream<List<int>> ytStream =
            yt.videos.streamsClient.get(streamInfo).asBroadcastStream();

        final BehaviorSubject<List<int>> controller =
            BehaviorSubject<List<int>>();

        if (onBytesReceived != null) {
          controller.addStream(ytStream);

          controller.listen((event) {
            onBytesReceived(streamInfo.size.totalBytes, file.lengthSync());
          });
        }

        IOSink fileStream = file.openWrite();

        await ytStream.pipe(fileStream);

        await fileStream.flush();
        await fileStream.close();

        controller.close();
        yt.close();
      }

      Uint8List data = await file.readAsBytes();

      if (progressDialog != null) {
        await progressDialog.dismissProgressDialog();
      }

      return data;
    } catch (e) {
      try {
        await file.delete();
      } catch (e) {
        print("ERROR downloadVideoFromYoutube file.delete() --> $e");
      }
      print("ERROR downloadVideoFromYoutube: $url --> $e");
      return null;
    }
  }

  @visibleForTesting
  static Future<MuxedStreamInfo> getStreamInfo(
    YoutubeExplode yt,
    String url,
  ) async {
    Video video = await yt.videos.get(url);

    StreamManifest manifest =
        await yt.videos.streamsClient.getManifest(video.id);

    int length = manifest.muxed.length;
    MuxedStreamInfo streamInfo = length > 1
        ? manifest.muxed.toList()[length - 1]
        : manifest.muxed.toList()[0];

    return streamInfo;
  }

  static Future<int?> getYoutubeDownLoadLength(String url) async {
    YoutubeExplode yt = YoutubeExplode();
    MuxedStreamInfo streamInfo = await getStreamInfo(yt, url);

    int lenght = streamInfo.size.totalBytes;

    yt.close();

    return lenght;
  }
}
